import { Directive, ElementRef, Injectable, InjectionToken, Input, NgModule, Renderer2 } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import { Subject } from 'rxjs/Subject';
import 'rxjs/add/observable/zip';
import 'rxjs/add/observable/from';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/take';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/filter';
import 'rxjs/add/operator/delay';
import 'rxjs/add/operator/skipWhile';

class HighlightService {
    /**
     * @param {?} options
     */
    constructor(options) {
        this.options = {
            theme: 'github',
            path: 'assets/lib/hljs',
            auto: true
        };
        this.loadScript$ = new Subject();
        this.highlighter$ = new Subject();
        this.ready$ = new Subject();
        this.options = Object.assign({}, this.options, options);
        /** Load hljs script and style only once */
        this.loadScript$
            .take(1)
            .do((renderer) => {
            this.renderer = renderer;
            this.loadScript();
            this.loadTheme();
        }).subscribe();
        /** highlight when script is ready */
        Observable.zip(this.ready$, this.highlighter$, (x, item) => {
            /** highlight one item or many elements */
            let codeElements = [];
            switch (item.type) {
                case 'all':
                    codeElements = item.el.querySelectorAll('pre code');
                    break;
                case '':
                    codeElements = [item.el];
                    break;
                default:
                    codeElements = item.el.querySelectorAll(this.highlight);
            }
            /** highlight all code elements */
            Observable.from(codeElements)
                .filter((code) => code.childNodes.length === 1 && code.childNodes[0].nodeName === '#text')
                .map((code) => {
                /** Highlight only if content is a plain text */
                const highlightedCode = hljs.highlightAuto(code.innerText.trim()).value;
                /** Render the highlighted code */
                if (highlightedCode !== code.innerHTML) {
                    this.renderer.setProperty(code, 'innerHTML', highlightedCode);
                }
            }).subscribe();
        }).subscribe();
    }
    /**
     * @param {?} renderer
     * @param {?} el
     * @param {?} type
     * @return {?}
     */
    highlight(renderer, el, type) {
        this.loadScript$.next(renderer);
        this.highlighter$.next({ el, type });
        if (isReady()) {
            this.ready$.next();
        }
    }
    /**
     * @return {?}
     */
    loadScript() {
        const /** @type {?} */ script = this.renderer.createElement('script');
        script.async = true;
        script.type = 'text/javascript';
        script.onload = () => {
            this.ready$.next();
        };
        script.src = `${this.options.path}/highlight.pack.js`;
        this.renderer.setAttribute(script, 'data-timestamp', new Date().getTime().toString());
        this.renderer.appendChild(document.body, script);
    }
    /**
     * @return {?}
     */
    loadTheme() {
        const /** @type {?} */ style = this.renderer.createElement('link');
        style.rel = 'stylesheet';
        style.type = 'text/css';
        style.href = `${this.options.path}/styles/${this.options.theme}.css`;
        this.renderer.appendChild(document.body, style);
    }
}
HighlightService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
HighlightService.ctorParameters = () => [
    null,
];
/**
 * @return {?}
 */
function isReady() {
    return typeof hljs !== 'undefined';
}

class HighlightDirective {
    /**
     * @param {?} el
     * @param {?} renderer
     * @param {?} hl
     */
    constructor(el, renderer, hl) {
        this.renderer = renderer;
        this.hl = hl;
        this.el = el.nativeElement;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.hl.highlight(this.renderer, this.el, this.highlight);
        /** Auto highlight on changes */
        if (this.hl.options.auto) {
            this.domObs = new MutationObserver(() => this.hl.highlight(this.renderer, this.el, this.highlight));
            this.domObs.observe(this.el, { childList: true, subtree: true });
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.hl.options.auto) {
            this.domObs.disconnect();
        }
    }
}
HighlightDirective.decorators = [
    { type: Directive, args: [{
                selector: '[highlight]'
            },] },
];
/**
 * @nocollapse
 */
HighlightDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
    { type: HighlightService, },
];
HighlightDirective.propDecorators = {
    'highlight': [{ type: Input },],
};

/**
 * @param {?} options
 * @return {?}
 */
function HighlightFactory(options) {
    return new HighlightService(options);
}
const OPTIONS = new InjectionToken('options');
class HighlightModule {
    /**
     * @param {?=} options
     * @return {?}
     */
    static forRoot(options) {
        return {
            ngModule: HighlightModule,
            providers: [
                { provide: OPTIONS, useValue: options },
                {
                    provide: HighlightService,
                    useFactory: HighlightFactory,
                    deps: [OPTIONS]
                }
            ]
        };
    }
}
HighlightModule.decorators = [
    { type: NgModule, args: [{
                declarations: [HighlightDirective],
                exports: [HighlightDirective]
            },] },
];
/**
 * @nocollapse
 */
HighlightModule.ctorParameters = () => [];

class HighlightUmdDirective {
    /**
     * @param {?} el
     * @param {?} renderer
     */
    constructor(el, renderer) {
        this.renderer = renderer;
        this.highlighter$ = new Subject();
        this.hlAuto = true;
        this.hlDelay = 200;
        this.el = el.nativeElement;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        let /** @type {?} */ codeElements = [];
        this.highlighter$
            .delay(this.hlDelay)
            .skipWhile(() => !isReady$1())
            .do(() => {
            switch (this.highlight) {
                case 'all':
                    codeElements = this.el.querySelectorAll('pre code');
                    break;
                case '':
                    codeElements = [this.el];
                    break;
                default:
                    codeElements = this.el.querySelectorAll(this.highlight);
            }
            Observable.from(codeElements)
                .take(1)
                .filter((code) => (code.childNodes.length === 1 && code.childNodes[0].nodeName === '#text'))
                .map((code) => {
                /**
                 * Highlight only if content is a plain text
                 */
                const highlightedCode = hljs.highlightAuto(code.innerText.trim()).value;
                /** Render the highlighted code */
                if (highlightedCode !== code.innerHTML) {
                    this.renderer.setProperty(code, 'innerHTML', highlightedCode);
                }
            }).subscribe();
        }).subscribe();
        this.highlighter$.next();
        /** Auto highlight on changes */
        if (this.hlAuto) {
            this.domObs = new MutationObserver(() => this.highlighter$.next());
            this.domObs.observe(this.el, { childList: true, subtree: true });
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.hlAuto) {
            this.domObs.disconnect();
        }
        this.highlighter$.unsubscribe();
    }
}
HighlightUmdDirective.decorators = [
    { type: Directive, args: [{
                selector: '[highlight]'
            },] },
];
/**
 * @nocollapse
 */
HighlightUmdDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
HighlightUmdDirective.propDecorators = {
    'highlight': [{ type: Input },],
    'hlAuto': [{ type: Input },],
    'hlDelay': [{ type: Input },],
};
/**
 * @return {?}
 */
function isReady$1() {
    return typeof hljs !== 'undefined';
}

class HighlightUmdModule {
}
HighlightUmdModule.decorators = [
    { type: NgModule, args: [{
                declarations: [HighlightUmdDirective],
                exports: [HighlightUmdDirective]
            },] },
];
/**
 * @nocollapse
 */
HighlightUmdModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { HighlightModule, HighlightUmdModule, HighlightUmdDirective as ɵe, HighlightDirective as ɵc, HighlightFactory as ɵa, OPTIONS as ɵb, HighlightService as ɵd };
//# sourceMappingURL=ngx-highlightjs.js.map
